/*
 * Copyright (c) 2022 The BytePlusRTC project authors. All Rights Reserved.
 * @brief RTSRoom
*/

#import <CoreMedia/CMTime.h>
#import "rtc/ByteRTCRtsDefines.h"

@class RTSRoom;

#pragma mark - RTSRoomDelegate
/** 
 * @type callback
 * @brief Audio room event callback interface
 */
@protocol RTSRoomDelegate <NSObject>
@optional

#pragma mark - Core Delegate Methods
 /** 
  * @type callback
  * @region Multi-room
  * @brief Callback on room state changes. Via this callback you get notified of room relating warnings,  errors and events. For example, the user joins the room, the user is removed from the room, and so on.
  * @param roomId  Room ID.
  * @param uid  User ID.
  * @param state Room state code. <br>
  *              + 0: Join room success. <br>
  *              + !0: Failed to join a room, abnormal exit, room-related warnings or errors.
  * @param extraInfo Extra information.
  *                 `join_type` indicates the type of room the user joins. `0` means the user joins the room for the first time, and `1` means the user rejoins the room. <br>
  *                 `elapsed` indicates the time interval from calling joinRoom{@link #RTSRoom#joinRoom} to successfully joining room, in ms.
  */
- (void)rtsRoom:(RTSRoom * _Nonnull)rtcRoom onRoomStateChanged:(NSString * _Nonnull)roomId
        withUid:(nonnull NSString *)uid state:(NSInteger)state
       extraInfo:(nonnull NSString *)extraInfo;

/** 
 * @type callback
 * @region Multiple rooms
 * @brief Leave the room successfully callback.   <br>
 *        After the user call leaveRoom{@link #RTSRoom#leaveRoom}, the SDK will stop sending/receiving broadcast messages in the room and, after releasing all relevant resources, notify the user via this callback that leaving the room was successful.<br>
 * @param rtsRoom rtsRoom object. <br>
 * @notes
 *        + After the user calls the leaveRoom{@link #RTSRoom#leaveRoom} method to leave the room, if destroy{@link #RTSRoom#destroy} is called to destroy the room instance or destroyRTS{@link #RTS#destroyRTS} method is called to destroy the RTS engine immediately, this callback event will not be received. <br>
 *        + If the app needs to use the system audio & video device after leaving the room, it is recommended to initialize the audio & video device after receiving this callback, otherwise the initialization may fail due to the SDK occupying the audio & video device. <br>
 */
 - (void)rtsRoomOnLeaveRoom:(RTSRoom *_Nonnull)rtsRoom;

/** 
 * @hidden
 * @deprecated since 3.46 and will be deleted in 3.52, use rtsRoom:onRoomStateChanged:withUid:state:extraInfo:{@link #RTSRoomDelegate#rtsRoom:onRoomStateChanged:withUid:state:extraInfo:} instead.
 * @type callback
 * @region Multiple rooms
 * @brief Error callback occurred.   <br>
 *        An error occurred during the SDK runtime. The SDK usually does not recover automatically, and the app needs to take action.
 * @param rtcRoom RTSRoom object.
 * @param errorCode Error code. See enumeration type ByteRTCErrorCode{@link #ByteRTCErrorCode}.
 */
- (void)rtsRoom:(RTSRoom * _Nonnull)rtcRoom onRoomError:(ByteRTSErrorCode)errorCode __deprecated_msg("Will be removed in new version");
/** 
 * @type callback
 * @region Multi-room
 * @brief When the remote user enters the room for the first time, or reconnects to the room after being disconnected, the other users in the room will receive this callback.
 * @param rtcRoom RTSRoom object. <br>
 * @param uid ID of the remote user who enters the room.
 * @param elapsed Reserved field, meaningless
 */
- (void)rtsRoom:(RTSRoom * _Nonnull)rtcRoom onUserJoined:(NSString * _Nonnull)uid
        elapsed:(NSInteger)elapsed;
/** 
 * @type callback
 * @brief This callback is triggered when a remote user is disconnected or turns invisible.
 * @param rtcRoom RTSRoom object. <br>
 * @param uid ID of the user who leaves the room, or switches to invisible. <br>
 * @param reason Reason to leave the room. See ByteRTCUserOfflineReason{@link #ByteRTCUserOfflineReason}.
 */
- (void)rtsRoom:(RTSRoom * _Nonnull)rtcRoom onUserLeave:(NSString * _Nonnull)uid
         reason:(ByteRTCUserOfflineReason)reason;

#pragma mark - Message Delegate Methods
 /** 
  * @type callback
  * @region Multi-room
  * @brief Receive this callback when you receive a broadcast text message from the room called sendRoomMessage:{@link #RTSRoom#sendRoomMessage:}.
  * @param rtcRoom RTSRoom object
  * @param uid Message sender ID.
  * @param message The content of the received text message.
  */
- (void)rtsRoom:(RTSRoom * _Nonnull)rtcRoom onRoomMessageReceived:(NSString * _Nonnull)uid
        message:(NSString * _Nonnull)message;
 /** 
  * @type callback
  * @region Multi-room
  * @brief Receive this callback when you receive a broadcast binary message sent by the room call sendRoomBinaryMessage:{@link #RTSRoom#sendRoomBinaryMessage:}.
  * @param rtcRoom RTSRoom object
  * @param uid Message sender ID.
  * @param message The binary message content received.
  */
- (void)rtsRoom:(RTSRoom * _Nonnull)rtcRoom onRoomBinaryMessageReceived:(NSString * _Nonnull)uid
        message:(NSData * _Nonnull)message;
 /** 
  * @type callback
  * @region Multi-room
  * @brief After sending a text or binary message to all users in the room, the message sender will receive a callback with the result of the message.
  * @param rtcRoom RTSRoom object.
  * @param msgid The ID of this message.
  * @param error  Message sending results. See ByteRTCRoomMessageSendResult{@link #ByteRTCRoomMessageSendResult}
  * @notes   <br>
  *         + You should call sendRoomMessage:{@link #RTSRoom#sendRoomMessage:} Send a mass text message to the room <br>
  *         + You should call sendRoomBinaryMessage:{@link #RTSRoom#sendRoomBinaryMessage:} Send a mass binary message to the room
  */
- (void)rtsRoom:(RTSRoom * _Nonnull)rtcRoom onRoomMessageSendResult:(int64_t)msgid
          error:(ByteRTCRoomMessageSendResult)error;

@end

#pragma mark - RTSRoom

/**
 * 
 * @type api
 */
BYTERTC_APPLE_EXPORT @interface RTSRoom : NSObject

/** 
 * @type callback
 */
@property(nonatomic, weak) id<RTSRoomDelegate> _Nullable delegate;

#pragma mark - Core Methods
/** 
 * @type api
 * @region multi-room
 * @brief Leaves and destroy the room instance created by calling createRoom:{@link #RTS#createRoom:}.
 */
- (void)destroy;
/** 
 * @hidden
 * @deprecated since 3.46 and will be deleted in 3.52.
 * @type api
 * @region multiple rooms
 * @brief Get room ID
 */
- (NSString *_Nullable)getRoomId __deprecated_msg("Will be removed in new version");
/** 
 * @type api
 * @region Multi-room
 * @brief By setting RTSRoomDelegate{@link #RTSRoomDelegate}, you can get infromed of events realted to this `RTSRoom` object.
 * @param roomDelegate Refer to RTSRoomDelegate{@link #RTSRoomDelegate}.
 * @return  <br>
 *        + 0: Success.
 *        + < 0 : Fail. See ByteRTCReturnStatus{@link #ByteRTCReturnStatus} for more details
 */
- (int)setRTSRoomDelegate:(id<RTSRoomDelegate> _Nullable)roomDelegate;

/** 
 * @type api
 * @region Multiple rooms
 * @brief Join the room. <br>
 *         After calling createRoom{@link #RTS#createRoom} to create an RTS room, you can call this API to enter the room and send/receive broadcast messages in the room.
 * @return  Method call result:  <br>
 *         + 0: Success <br>
 *         + -2: Already in the room. <br>
 *         + -3: No room is created.
 * @notes   <br>
 *         + You should first call login{@link #RTS#login} to log in to the RTS system and then join the room. <br>
 *         + Local users will receive rtsRoom:onRoomStateChanged:withUid:state:extraInfo:{@link #RTSRoomDelegate#rtsRoom:onRoomStateChanged:withUid:state:extraInfo:} callback notification after calling this method to join the room successfully. <br>
 *         + After the user successfully joins the room, the SDK may lose connection to the server in case of poor local network conditions, and the SDK will automatically reconnect at this time. After successful reconnection, you will receive a callback notification from rtsRoom:onRoomStateChanged:withUid:state:extraInfo:{@link #RTSRoomDelegate#rtsRoom:onRoomStateChanged:withUid:state:extraInfo:} locally. If the user is set to visiable, the remote user will receive rtsRoom:onUserJoined:elapsed:{@link #RTSRoomDelegate#rtsRoom:onUserJoined:elapsed:} callback notification. 
 */
- (int)joinRoom;

/** 
  * @type api
  * @region  room management
  * @brief Leave RTC room.   <br>
  *        The user calls this method to leave the room, ending real-time message communication and releasing all communication-related resources. <br>
  * @return   <br>
  *         + 0: Success <br>
  *         + < 0: Failure <br>
  * @notes   <br>
  *        + When a visible user leaves the room, others in the room will receive rtsRoom:onUserLeave:reason:{@link #RTSRoomDelegate#rtsRoom:onUserLeave:reason:}. <br>
  *        + This method is an asynchronous operation, and the call returns without actually exiting the room. After you actually exit the room, you will receive a callback notification from rtsRoomOnLeaveRoom:{@link #RTSRoomDelegate#rtsRoomOnLeaveRoom:} locally. <br>
  *        + If the engine is destroyed immediately after this method is called, the SDK will not be able to trigger the rtsRoomOnLeaveRoom:{@link #RTSRoomDelegate#rtsRoomOnLeaveRoom:} callback. <br>
  */
- (int)leaveRoom;

#pragma mark - Message Methods

 /** 
  * @type api
  * @region Multi-room
  * @brief Send text messages to all other users in the room.
  * @param message   <br>
  *        The content of the text message sent. <br>
  *         Message does not exceed 62KB.
  * @return The number of the message sent this time is incremented from 1.
  * @notes   <br>
  *       + Before sending an in-room text message, you must call joinRoom{@link #RTSRoom#joinRoom} to join the room. <br>
  *       + After calling this function, you will receive an rtsRoom:onRoomMessageSendResult:error:{@link #RTSRoomDelegate#rtsRoom:onRoomMessageSendResult:error:} callback to notify the sender of the success or failure of the message; <br>
  *       + If the text message is sent successfully, all remote users in the room will receive rtsRoom:onRoomMessageReceived:message:{@link #RTSRoomDelegate#rtsRoom:onRoomMessageReceived:message:} callback.
  */
- (int64_t)sendRoomMessage:(NSString * _Nonnull)message;
 /** 
  * @type api
  * @region multiroom
  * @brief Sends binary messages to all other users in the room.
  * @param message   <br>
  *        The binary broadcast message sent by the user <br>
  *        The message does not exceed 46KB.
  * @return The number of the message sent this time is incremented from 1.
  * @notes   <br>
  *       + Before broadcasting binary messages in the room, you must call joinRoom{@link #RTSRoom#joinRoom} to join the room. <br>
  *       + After calling this function, you will receive an rtsRoom:onRoomMessageSendResult:error:{@link #RTSRoomDelegate#rtsRoom:onRoomMessageSendResult:error:} callback to notify the sender of the success or failure of the message; <br>
  *       + If the binary message is successfully sent, all users in the room will receive rtsRoom:onRoomBinaryMessageReceived:message:{@link #RTSRoomDelegate#rtsRoom:onRoomBinaryMessageReceived:message:} callback.
  */
- (int64_t)sendRoomBinaryMessage:(NSData *  _Nonnull)message;

@end
